#-*- coding: utf-8 -*-

from django.contrib import admin
from django.urls import path
from django.conf.urls import url, include

import mediautils
from mediautils.views import del_photo


urlpatterns = [
    url(r'^del-photo/(?P<pk>\d+)/$', mediautils.views.del_photo, name='del_photo'),
    ]



from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect, get_object_or_404
from django.urls import reverse

from infscroll.utils import get_pagination
from infscroll.views import more_items

from social_layer.posts.forms import PostForm
from social_layer.posts.models import Post, PostMedia
from social_layer.utils import get_social_profile
from social_layer.mediautils.utils import handle_upload_file


def new_post(request):
    """ manages the creation of new user generated content """
    if not request.user.is_authenticated:
        return redirect(reverse('social_layer:social_login'))
    if request.method == "POST":
        form = PostForm(request.POST, request.FILES,
                        initial={'owner': request.user})
        if form.is_valid():
            post = form.save(commit=False)
            post.owner = get_social_profile(request)
            post.save()
            if (form.allow_media and len(request.FILES) > 0):
                media = handle_upload_file(file_post=request.FILES.get('media'),
                                            quality=1,
                                            Model=PostMedia,
                                            extra_args={'post': post})
            return redirect(reverse('social_layer:posts_feed'))
    else:
        form = PostForm()
    return render(request, 'social_layer/posts/new_post.html', {'form': form,})


def posts_feed(request):
    """ The main list of posts """
    post_list = Post.objects.all().order_by('-id')
    paginated = get_pagination(request, post_list)
    data = {
        'more_posts_url': reverse('social_layer:more_posts'),
        'form': PostForm(),
        }
    data.update(paginated)
    return render(request, 'social_layer/posts/posts_feed.html', data)

def more_posts(request):
    """ dynamic load posts using the django-infinite-scroll module.
    """
    post_list = Post.objects.all().order_by('-id')
    return more_items(request, post_list,
                      template='social_layer/posts/more_posts.html')

def view_post(request, pk, template='social_layer/posts/view_post.html'):
    """ The main list of posts """
    post = get_object_or_404(Post, pk=pk)
    data = {
        'post': post,
        'comment_section': post.comments,
        }
    return render(request, template, data)


def delete_post(request, pk):
    """ The main list of posts """
    post = get_object_or_404(Post, pk=pk, owner__user=request.user)
    post.delete()
    return redirect(reverse('social_layer:posts_feed'))



from django import forms
from django.conf import settings
from django.utils.translation import gettext as _
from social_layer.posts.models import Post, PostMedia
from social_layer.mediautils.utils import handle_upload_file
    
class PostForm(forms.ModelForm):
    text = forms.CharField(required=False,
                label=_("Text"),
                widget=forms.Textarea(attrs={
                                        "class": "comment-textarea",
                                        "rows": 4
                                        }))
    allow_media = getattr(settings, 'SOCIAL_ALLOW_MEDIA_POSTS', True)
    
    class Meta:
        model = Post
        fields = ['text',]

    def __init__(self, *args, **kwargs):
        """ PostForm __init__ method
        decides if should show "media" field, based on SOCIAL_ALLOW_MEDIA_POSTS
        """
        super(PostForm, self).__init__(*args, **kwargs)
        if self.allow_media:
            self.fields['media'] = forms.FileField(required=False)

    def clean_text(self):
        """ if there is no text, we must at least have a file. 
        Otherwise is not valid.
        """
        if (not self.cleaned_data['text']
            and len(self.files) < 1):
            raise forms.ValidationError(_('Say something'))
        return self.cleaned_data['text']
        
    def clean_media(self):
        """ raise invalid if media is not allowed
        But this will probably not fire up because when so, the field 
            will be disabled
        """
        if not self.allow_media:
            raise forms.ValidationError(_('Posting media is not allowed!'))
        return self.cleaned_data['media']
        



import os
from uuid import uuid4
from django.contrib.auth.decorators import login_required
from django.http import Http404
from django.shortcuts import render, redirect
from django.urls import reverse

# Create your views here.
from social_layer.mediautils.models import Media
from social_layer.mediautils.utils import rotate_image

@login_required
def del_photo(request, pk, MediaModel=Media):
    """ deletes an image
    To be used wraped in another view
    """
    try:
        media = MediaModel.objects.get(id=pk, owner__user=request.user)
    except MediaModel.DoesNotExist:
        raise Http404()
    media.delete()
    return redirect(reverse('root'))

@login_required
def mediautils_rotate_photo(request, pk, direct='left', MediaModel=Media):
    """ Rotate an image
    To be used wraped in another view
    """
    try:
        media = MediaModel.objects.get(id=pk, owner__user=request.user)
    except MediaModel.DoesNotExist:
        raise Http404()
    for field in ['media_file', 'media_thumbnail']:
        file_field = getattr(media, field, None)
        orig_name = file_field.name
        img_file = file_field.path
        rotate_image(img_file, direct)
        base_name = os.path.basename(img_file)
        fname, ext = os.path.splitext(base_name)
        random_name = uuid4().hex
        new_path = img_file.replace(fname, random_name)
        new_name = orig_name.replace(fname, random_name)
        os.rename(img_file, new_path)
        if field == 'media_file':
            media.media_file.name = new_name
        elif field == 'media_thumbnail':
            media.media_thumbnail.name = new_name
        media.save()
        #media.refresh_from_db()
        #assert os.path.isfile(getattr(media, field, None).path)
        #assert new_name in getattr(media, field, None).path
        #assert new_name in getattr(media, field, None).url
    return media




# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

from django.contrib import admin

# Register your models here.
from social_layer.comments.models import Comment, CommentSection, LikeComment
from social_layer.notifications.models import Notification
from social_layer.profiles.models import SocialProfile, SocialProfilePhoto
from social_layer.posts.models import Post, PostMedia

admin.site.register(SocialProfile)
admin.site.register(SocialProfilePhoto)
admin.site.register(Notification)
admin.site.register(Comment)
admin.site.register(CommentSection)
admin.site.register(LikeComment)
admin.site.register(Post)
admin.site.register(PostMedia)



# Generated by Django 3.2.10 on 2021-12-22 14:08

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import social_layer.mediautils.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('social_layer', '0003_likecomment'),
    ]

    operations = [
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(blank=True, null=True)),
                ('date_time', models.DateTimeField(default=django.utils.timezone.localtime)),
            ],
        ),
        migrations.AlterField(
            model_name='commentsection',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='comment_section_owner', to='social_layer.socialprofile'),
        ),
        migrations.AlterField(
            model_name='likecomment',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likecomment_user_likecomments', to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='PostMedia',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('media_file', models.FileField(max_length=10485760, upload_to=social_layer.mediautils.models.choose_upload_media_to)),
                ('media_thumbnail', models.FileField(blank=True, max_length=10485760, null=True, upload_to=social_layer.mediautils.models.choose_upload_media_thumb_to)),
                ('content_type', models.CharField(blank=True, max_length=127, null=True)),
                ('orientation', models.CharField(default='portrait', max_length=10)),
                ('format_tries', models.IntegerField(default=0)),
                ('formated', models.BooleanField(default=False)),
                ('md5_hash', models.CharField(blank=True, max_length=32, null=True)),
                ('post', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='social_layer.post')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='post',
            name='comments',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='post_commentsection', to='social_layer.commentsection'),
        ),
        migrations.AddField(
            model_name='post',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='post_commentsection_owner', to='social_layer.socialprofile'),
        ),
        migrations.CreateModel(
            name='LikePost',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('like', models.BooleanField(default=True)),
                ('date_time', models.DateTimeField(default=django.utils.timezone.localtime)),
                ('comment_section', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='like_commentsection', to='social_layer.commentsection')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likecomment_user_likeposts', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import include
from django.urls import path
from django.conf import settings
#from django.conf.urls.static import static


import social_layer

#from social_layer.views.comments import *

from social_layer.profiles.views import *
from social_layer.comments.views import *
from social_layer.notifications.views import *
from social_layer.posts.views import *

app_name = 'social_layer'

urlpatterns = [
    ##############
        # profile
        path('profile/', 
             social_layer.profiles.views.setup_profile, name='setup_profile'),
        path('profile/<int:pk>/',
             social_layer.profiles.views.view_profile, name='view_profile'),
        path('community/',
             social_layer.profiles.views.list_profiles, name='list_profiles'),
        path('social-login/',
             social_layer.profiles.views.social_login, name='social_login'),
        path('delete-profile-pic/',
             social_layer.profiles.views.delete_profile_photo,
             name='delete_profile_photo'),
        # comments
        path('comments/',
             social_layer.comments.views.comment_section, name='comment_section'),
        path('comments/<int:pk>/',
             social_layer.comments.views.comment_section, name='comment_section'),
        path('reply-comment/<int:pk>/',
             social_layer.comments.views.reply_comment, name='reply_comment'),
        path('del-comment/<int:pk>/',
             social_layer.comments.views.delete_comment, name='delete_comment'),
        # likes
        path('like-comment/<int:pk>/<slug:didlike>/',
             social_layer.comments.views.like_comment, name='like_comment'),
        path('like-post/<int:pk>/<slug:didlike>/',
             social_layer.comments.views.like_post, name='like_post'),
        # notifications
        path('notifications/',
             social_layer.notifications.views.view_notifications, name='view_notifications'),
        path('notifications/adm-send/<int:pk>/',
             social_layer.notifications.views.admin_send_notification, name='admin_send_notification'),
        #posts
        path('new-post/', social_layer.posts.views.new_post, name='new_post'),
        path('feed/', social_layer.posts.views.posts_feed, name='posts_feed'),
        path('more-posts/', social_layer.posts.views.more_posts, name='more_posts'),
        path('post/<int:pk>/', social_layer.posts.views.view_post, name='view_post'),
        path('delete-post/<int:pk>/', social_layer.posts.views.delete_post, name='delete_post'),
        ]




# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

from django.shortcuts import render, redirect

# Create your views here.
from time import time
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, Http404
from django.utils.translation import gettext as _
from django.urls import reverse
from social_layer.comments.models import CommentSection, Comment, LikeComment, LikePost
from social_layer.notifications.models import Notification
from social_layer.utils import get_social_profile


def comment_section(request, pk=None):
    """ renders a full comment section
    and also stores a new comment
    """
    sprofile = get_social_profile(request)
    if pk:
       section = CommentSection.objects.get(id=pk)
    else:
       section = CommentSection.objects.create(owner=sprofile)
    if request.method == "POST":
        text = request.POST.get("text", '')
        reply_to = request.POST.get("reply_to", None)
        if reply_to:
            reply_to = Comment.objects.filter(pk=reply_to).first()
        comment = Comment.objects.create(author=sprofile,
                                         comment_section=section,
                                         text=text,
                                         reply_to=reply_to)
        parties = []
        if section.owner:
            parties.append(section.owner.user)
        if reply_to:
            jump_to = reply_to.pk
            parties.append(reply_to.author.user)
        else:
            jump_to = comment.pk
            parties.append(comment.author.user)
        if jump_to != None:
            comment_url = section.get_url()+'?show-comments#comment_'+str(jump_to)
            message_list = [sprofile.nick,
                            '<a href="{}" class="alink">'.format(comment_url),
                            _("wrote a comment"),
                            '</a>']
            notif_text = ' '.join(message_list)
            notify_parties(parties, notif_text, do_not=[request.user])
        else:
            comment_url = section.get_url()+'?show-comments'
        return redirect(comment_url)
    data = {
        'comment_section': section,
        }
    return render(request, 'social_layer/comments/comments_view.html', data)


@login_required
def reply_comment(request, pk):
    """ Called when posting a reply to a comment """
    sprofile = get_social_profile(request)
    reply_to = Comment.objects.filter(id=pk).first()
    if not reply_to:
        raise Http404()
    section = reply_to.comment_section
    if request.method == "POST":
        text = request.POST.get("text", '')
        comment = Comment.objects.create(author=sprofile,
                                         comment_section=section,
                                         text=text,
                                         reply_to=reply_to)
        if '?' in section.get_url():
            spacer = '&'
        else:
            spacer = '?'
        comment_url = ''.join([
                            section.get_url(),
                            spacer,
                            'show-comments',
                            '&t='+str(int(time())),
                            '#comment_'+str(reply_to.pk),
                            ])
        message_list = [sprofile.nick,
                        '<a href="{}" class="alink">'.format(comment_url),
                        _("wrote a comment"),
                        '</a>',
                        text[0:10]+'...',]
        notif_text = ' '.join(message_list)
        parties = [reply_to.author.user,]
        if section.owner:
            parties.append(section.owner.user)
        notify_parties(parties, notif_text, do_not=[request.user])
        
        return redirect(comment_url)
    return redirect(section.get_url())

@login_required
def delete_comment(request, pk):
    """ Deletes a comment """
    sprofile = get_social_profile(request)
    comment = Comment.objects.filter(id=pk).first()
    if not comment:
        raise Http404()
    section = comment.comment_section
    if (sprofile == comment.author
        or section.owner_can_delete and sprofile == section.owner
        or request.user.is_superuser):
        comment.delete()
    return redirect(section.get_url())


def notify_parties(parties, text, do_not=[]):
    """ Creates a notification to people interested on it """
    for party in set(parties):
        if (party and party not in do_not):
            Notification.objects.create(to=party,
                                        text=text)

@login_required
def like_comment(request, pk, didlike):
    """ when someone likes a comment """
    comment = Comment.objects.filter(pk=pk).first()
    if not comment:
        raise Http404()
    liked = LikeComment.objects.filter(user=request.user, comment=comment).first()
    if not liked:
        liked = LikeComment.objects.create(user=request.user, comment=comment)
    liked.like = bool(didlike == 'like')
    liked.save()
    comment.updt_counters()
    section = comment.comment_section
    comment.refresh_from_db()
    if 'ajx' in request.GET.keys():
        if liked.like:
            count = comment.count_likes
        else:
            count = comment.count_dislikes
        return HttpResponse(count, content_type='text/txt')
    else:
        return redirect(section.get_url())
    
@login_required
def like_post(request, pk, didlike):
    """ when someone likes a comment_section """
    section = CommentSection.objects.filter(pk=pk).first()
    if not section:
        raise Http404()
    liked = LikePost.objects.filter(user=request.user, comment_section=section).first()
    if not liked:
        liked = LikePost.objects.create(user=request.user, comment_section=section)
    liked.like = bool(didlike == 'like')
    liked.save()
    section.updt_counters()
    if 'ajx' in request.GET.keys():
        if liked.like:
            count = section.count_likes
        else:
            count = section.count_dislikes
        return HttpResponse(count, content_type='text/txt')
    else:
        return redirect(section.get_url())




import logging
from time import sleep
from django.core.management.base import BaseCommand, CommandError
from social_layer.tasks import format_medias

logger = logging.getLogger(__name__)

class Command(BaseCommand):
    help = 'Task that format the media files used by mediaultils app'

    def add_arguments(self, parser):
        parser.add_argument('--daemon', action='store_true')
        parser.add_argument('--interval', type=int, default=40)

    def handle(self, *args, **kwargs):
        """ Run the format media task. The options are:
        --daemon - this means that the it will run forever.
        --interval - sets the amount of sleep time between loops, if
                    'daemon' mode is set.
        """
        logger.info("Task format_media... Running.")
        while True:
            format_medias()
            if not kwargs.get('daemon'):
                break
            sleep(kwargs.get('interval'))





from django.contrib import admin

# Register your models here.
from social_layer.comments.models import *


admin.site.register(Comment)
admin.site.register(CommentSection)
admin.site.register(LikeComment)



# -*- coding: utf-8 -*-
import json
from uuid import uuid4
import mimetypes
import cv2
import os
import sys
import shutil
import random
from PIL import (Image, ImageDraw, ImageFont,
                 ImageSequence, UnidentifiedImageError)

from django.conf import settings
import logging
import hashlib
import traceback

from django.core.files.base import ContentFile
from django.core.files import File
from social_layer.mediautils.models import Media

TEMP_FILE_DIR = '/tmp/'
logger = logging.getLogger(__name__)

def cropa_imagem(img_file, larger=480, smaller=360, quality=1):#3x4 # 
    """ reduce image size """
    larger *= quality
    smaller *= quality
    cropamap = {
        'landscape': [larger, smaller],
        'portrait':  [smaller, larger],
        }
    try:
        orient = get_img_orientation(img_file)
        # convert to jpg
        img = Image.open(img_file)
        img = img.convert('RGBA')
        background = Image.new("RGBA", img.size, "WHITE")
        background.paste(img, (0, 0), img) 
        img = background.convert('RGB')
        img.save(img_file, "JPEG", optimize=True, quality=80)

        fd_img = open(img_file, 'rb')
        img = Image.open(fd_img)
        img.thumbnail(cropamap[orient], Image.ANTIALIAS)
        img.save(img_file, "JPEG", optimize=True, quality=80)
        fd_img.close()
        ret = cropamap[orient]
    except Exception as e:
        ret = 'landscape'
        logger.error(e)
    return ret

def cropa_video(video_file, larger=320, smaller=240, quality=1):
    """ reduce video size """
    larger *= quality
    smaller *= quality
    try:
        random_hex = uuid4().hex
    except: #python2
        random_hex = uuid4().get_hex()
    file_temp = '/tmp/'+random_hex+'.mp4'
    try:
        #+ ' -s '+str(larger) +'x'+ str(smaller)+' '
        #+ ' -c:v libx264 -preset slow -an -b:v 370K '
        #+ ' -c:a aac -movflags +faststart '
        #+ ''' -vf "scale='min('''+str(larger) +''',iw)':'min('''+str(smaller) +''',ih)'" '''
        cmd = ('/usr/bin/ffmpeg -hide_banner -loglevel error -i '
                    + video_file
                    + ' -vf scale='+str(larger)+':-2 '
                    + file_temp)
        #print(cmd)
        os.system(cmd)
        if os.path.isfile(file_temp):
            shutil.move(file_temp, video_file)
            return True
    except Exception as e:
        logger.error(e)
    return False

def rotate_image(img_file, direct='left'):
    """ rotate image """
    if direct=='left':
        angle = 90
    else:
        angle = -90
    fd_img = open(img_file, 'rb')
    img = Image.open(fd_img)
    rot = img.rotate(angle, expand=1, resample=Image.BICUBIC)
    fd_img.close()
    rot.save(img_file)
    
    
def get_img_orientation(img_file):
    """ Get the orientation of image """
    try:
        img = cv2.imread(img_file)
        height, width, channels = img.shape
        h, w = img.shape[:2]
        if h > w:
            return 'portrait'
        else:
            return 'landscape'
    except Exception as e:
        logger.error(e)
        return 'portrait'


def md5sum(filename, blocksize=65536):
    """ Get md5sum from file """
    hashe = hashlib.md5()
    with open(filename, "rb") as f:
        for block in iter(lambda: f.read(blocksize), b""):
            hashe.update(block)
        f.close()
    return hashe.hexdigest()


def convert_tojpeg(tempo_file):
    """ convert image to jpeg """
    try:
        img = Image.open(tempo_file)
        img = img.convert('RGBA')
        background = Image.new("RGBA", img.size, "WHITE")
        background.paste(img, (0, 0), img) 
        img = background.convert('RGB')
        img.save(tempo_file, "JPEG", optimize=True, quality=80)
    except Exception as e:
        print('ERROR: convert_tojpeg', e)
        logger.error(e)

def convert_towebp(tempo_file):
    """ convert image to webp
    requires libwebp-dev
    """
    try:
        img = Image.open(tempo_file)
        img = img.convert('RGBA')
        background = Image.new("RGBA", img.size, "WHITE")
        background.paste(img, (0, 0), img) 
        img = background.convert('RGB')
        img.save(tempo_file, "WEBP")
    except Exception as e:
        logger.error(e)

def handle_upload_file(file_post=None,
                     Model=None,
                     extra_args={},
                     quality=1):
    """ handle a file upload """
    retorno = None
    # creates a temp file
    try:
        random_hex = uuid4().hex
    except: #python2
        random_hex = uuid4().get_hex()
    # Save temp file in /tmp/
    tempo_file = TEMP_FILE_DIR + random_hex
    with open(tempo_file, 'wb') as temp_file:
        for chunk in file_post.chunks():
            temp_file.write(chunk)
        temp_file.close()
    extension = mimetypes.guess_extension(file_post.content_type, strict=True)
    if extension == 'jpe':
        # guess_extension eh bugado
        extension = 'jpg'
    if not extension:
        extension = ''
    # get a md5 from the file, and this will be it's new name
    md5_hash = md5sum(tempo_file)
    filename = ''.join([md5_hash, extension])
    # rename temp with extension
    tempo_file_ext = ''.join([tempo_file, extension])
    os.rename(tempo_file, tempo_file_ext)
    tempo_file = tempo_file_ext
    # Here we have a file in /tmp/ with name md5.extension
    mime_type = file_post.content_type
    if not mime_type:
        mime_type = []
    retorno = handle_generic_file(tempo_file,
                                file_post,
                                filename,
                                quality=quality,
                                Model=Model,
                                extra_args=extra_args,
                                md5_hash=md5_hash)
    return retorno

def handle_generic_file(tempo_file,
                        file_post,
                        filename,
                        quality=1,
                        Model=Media,
                        extra_args={},
                        md5_hash=None):
    """ handle a generic file uplod and format it accordint to it's needs """
    media = Model(**extra_args)
    mime_type = file_post.content_type
    if 'image/gif' in mime_type:
        tipo = 'gif'
    elif 'image' in mime_type:
        tipo = 'image'
    elif 'video' in mime_type:
        tipo = 'video'
    elif 'audio' in mime_type:
        tipo = 'audio'
    elif 'application/octet-stream' in mime_type:
        tipo = ('gif' if check_if_img(tempo_file) else 'file')
    else:
        tipo = 'file'
    tipos_extensions = {
        'image': ['.jpg', '.webp'],
        'gif': ['.gif', '.webp'],
        }
    if tipo in tipos_extensions.keys():
        mainext, thumbext = tipos_extensions.get(tipo, ['.jpg', '.webp'])
        fname_spl = filename.split('.')
        if len(fname_spl) > 0:
            first_name = fname_spl[0]
            filename = first_name+mainext
            thumbname = first_name+thumbext
        else:
            filename = filename+mainext
            thumbname = filename+thumbext
    else:
        thumbname = filename
    # convert to jpe
    media.md5_hash = md5_hash
    media.content_type = mime_type
    if (('image/gif' in mime_type) or
        ('application/octet-stream' in mime_type)):
        arruma_gif(tempo_file, mime_type)
    elif 'image' in mime_type:
        convert_tojpeg(tempo_file)
    foto_file = open(tempo_file, 'rb')
    media.media_file.save(filename, File(foto_file))
    foto_file.close()
    # extract a thumbnail depending on filetype
    if tipo == 'image': # , 'gif' # we will treat gifs differently
        thumb_file = open(tempo_file, 'rb')
        media.media_thumbnail.save(thumbname, File(thumb_file))
        thumb_file.close()
        # Crop the imagem of thumbnail
        cropa_imagem(media.media_thumbnail.path, quality=quality)
        convert_towebp(media.media_thumbnail.path)
    elif tipo == 'gif':
        media.media_thumbnail = media.media_file
    elif tipo == 'video':
        media.save()
        get_thumb_from_video(media)
    media.save()
    # Delete temp file
    os.remove(tempo_file)
    return media

def arruma_gif(tempo_file, mime_type):
    """ fix gifs that came with mime 'application/octet-stream'
    """
    if ('application/octet-stream' in mime_type):
        # se for octet stream, tem q converter ele pra gif
        os.system('convert '+tempo_file+ ' ' + tempo_file+'.gif')
        if os.path.isfile(tempo_file+'.gif'):
            shutil.move(tempo_file+'.gif', tempo_file)

def get_thumb_from_video(video):
    """ extract thumbnail from videos.
    requires ffmpeg
    """
    try:
        random_hex = uuid4().hex
    except: #python2
        random_hex = uuid4().get_hex()
    thumbnail_temp = '/tmp/'+random_hex+'.jpg'
    cmd = ('/usr/bin/ffmpeg -hide_banner -loglevel panic -y -ss 0 -i '
                + video.media_file.path
                + ' -frames:v 1 -s 400x300 '
                + thumbnail_temp)
    os.system(cmd)
    if os.path.isfile(thumbnail_temp):
        #thumbnail
        thumb_file = open(thumbnail_temp, 'rb')
        video.media_thumbnail.save(os.path.basename(video.media_file.name), File(thumb_file))
        thumb_file.close()
        os.remove(thumbnail_temp)

def check_if_img(file_path:str)->bool:
    """ check if file given by path is an actual IMAGE file
    """
    try:
        media_file = Image.open(file_path)
    except (UnidentifiedImageError, FileNotFoundError) as e:
        return False
    return (len(list(ImageSequence.Iterator(media_file))) > 0)



import os
from django.db import models
from django.utils import timezone

# Create your models here.
def _choose_upload_media_to(instance, filename, dir_prefix):
    """ choose directory to upload files
    """
    if len(filename) > 5:
        filepath = "{}/{}/{}".format(filename[:2], filename[:4], filename)
    else:
        filepath = filename
    name = dir_prefix + filepath
    return name


def choose_upload_media_to(instance, filename):
    """ where to upload the main file """
    return _choose_upload_media_to(instance, filename, 'files/sl/')


def choose_upload_media_thumb_to(instance, filename):
    """ where to upload the thumbnail file """
    return _choose_upload_media_to(instance, filename, 'files/sl/thumbs/')


class Media(models.Model):
    """ abstract class for media objects """
    media_file = models.FileField(upload_to=choose_upload_media_to,
                                                    max_length=10485760)
    media_thumbnail = models.FileField(upload_to=choose_upload_media_thumb_to,
                                                    max_length=10485760,
                                                    null=True, blank=True)
    content_type = models.CharField(max_length=127, null=True, blank=True,
                                    default='application/octet-stream')
    orientation = models.CharField(max_length=10, default='portrait')

    format_tries = models.IntegerField(default=0)
    formated = models.BooleanField(default=False)
    md5_hash = models.CharField(max_length=32, null=True, blank=True)

    class Meta():
        abstract = True

    def save(self, *args, **kwargs):
        from social_layer.mediautils.utils import get_img_orientation
        super(Media, self).save(*args, **kwargs)
        # Try to guess image orientation
        if 'image/' in self.content_type:
            self.orientation = get_img_orientation(self.media_file.path)
        elif self.media_thumbnail:
            self.orientation = get_img_orientation(self.media_thumbnail.path)
        super(Media, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        attr_list = [ 'media_file', 'media_thumbnail' ]
        for attr in attr_list:
            arquivo = getattr(self, attr, None)
            if (arquivo and os.path.isfile(arquivo.path)):
                os.remove(arquivo.path)
        super(Media, self).delete(*args, **kwargs)



# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase, Client
## Create your tests here.
from uuid import uuid4
from base64 import b64decode
import os
import random
import shutil
from django.conf import settings
from django.urls import reverse
from django.contrib.auth.models import User
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test.utils import override_settings
##
from social_layer.comments.models import (CommentSection,
                                 Comment,
                                 LikeComment)
from social_layer.profiles.models import SocialProfile
from social_layer.notifications.models import Notification
from social_layer.posts.models import Post, PostMedia
from social_layer.mediautils.tests import small_video, small_image
from social_layer.tasks import format_medias
import logging

#@override_settings(LOGGING={})
@override_settings(MEDIA_ROOT='/tmp/media_test_{}/'.format(uuid4().hex))
class SocialLayerTestCase(TestCase):
    """ Test Cases for the Social Media application.
    currently covering around 80% of the code.
    """
    def setUp(self):
        """ create two users and a comment section """
        super(SocialLayerTestCase, self).setUp()
        self.bob = User.objects.create(username='Bob')
        self.alice = User.objects.create(username='Alice')
        # log once to create social_profile
        self.bob_sprofile = SocialProfile.objects.create(user=self.bob)
        self.alice_sprofile = SocialProfile.objects.create(user=self.alice)
        
        self.comment_section = CommentSection.objects.create(
                                                    owner=self.alice_sprofile)
        # disable logging
        logging.disable(logging.INFO)

    def tearDown(self):
        """ removes created files """
        shutil.rmtree(settings.MEDIA_ROOT, ignore_errors=True)

    def test_write_comment(self):
        """ Bob writes a comment at Alice's page. """
        client = Client()
        client.force_login(self.bob)
        section = CommentSection.objects.create(owner=self.alice_sprofile)
        response = client.get(section.get_url())
        self.assertIn(self.alice.username, str(response.content))
        post_data = {
            'text': uuid4().hex,
            }
        response = client.post(section.get_url(), post_data, follow=True)
        self.assertIn(self.bob_sprofile.nick, str(response.content))
        self.assertIn(post_data['text'], str(response.content))
        notif = Notification.objects.get(to=self.alice)
        self.assertFalse(notif.read)
        
    def test_comments_and_replies(self):
        """ Bob and Alice comments at each other's comments. """
        section = CommentSection.objects.create(owner=self.alice_sprofile)
        messages = []
        for user in [self.bob, self.alice]:
            client = Client()
            client.force_login(user)
            for i in range(5):
                post_data = {'text': uuid4().hex,}
                response = client.post(section.get_url(), post_data, follow=True)
                self.assertIn(post_data['text'], str(response.content))
                messages.append(post_data['text'])
        for user in [self.bob, self.alice]:
            client = Client()
            client.force_login(user)
            for i in range(10):
                post_data = {'text': uuid4().hex,}
                a_comment = random.choice(Comment.objects.all())
                url = reverse('social_layer:reply_comment',
                              kwargs={'pk': a_comment.pk})
                response = client.post(url, post_data, follow=True)
                self.assertIn(post_data['text'], str(response.content))
                messages.append(post_data['text'])
        response = client.get(section.get_url()+'?show-comments')
        for msg in messages:
            self.assertIn(msg, str(response.content))

    def test_repeat_comment(self):
        """ Ensures that a comment can't be made twice """
        client = Client()
        client.force_login(self.bob)
        section = CommentSection.objects.create(owner=self.alice_sprofile)
        response = client.get(section.get_url())
        self.assertIn(self.alice.username, str(response.content))
        post_data = {
            'text': uuid4().hex,
            }
        for i in range(0,3):
            response = client.post(section.get_url(), post_data, follow=True)
        self.assertIn(self.bob_sprofile.nick, str(response.content))
        self.assertIn(post_data['text'], str(response.content))
        comments = Comment.objects.filter(comment_section=section)
        self.assertEqual(comments.count(), 1)
        notifs = Notification.objects.filter(to=self.alice)
        self.assertEqual(notifs.count(), 1)
        self.assertFalse(notifs[0].read)

    def test_community(self):
        """ test the list profiles view """
        client = Client()
        client.force_login(self.bob)
        response = client.get(reverse('social_layer:list_profiles'))
        self.assertIn(self.bob_sprofile.nick, str(response.content))
        self.assertEqual(response.status_code, 200)
        
    def test_view_profile(self):
        """ test the profile page """
        client = Client()
        client.force_login(self.bob)
        response = client.get(self.alice_sprofile.get_url())
        self.assertIn(self.alice_sprofile.nick, str(response.content))
        self.assertEqual(response.status_code, 200)
        
       
    def test_setup_profile(self):
        """ check the profile setup page """
        client = Client()
        client.force_login(self.bob)
        url = reverse('social_layer:setup_profile')
        response = client.get(url)
        self.assertEqual(response.status_code, 200)
        post_data = {
            'nick': 'Bob Tester',
            'phrase': 'Testing this',
            'receive_email': 'on',
            }
        response = client.post(url, post_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.bob_sprofile.refresh_from_db()
        self.assertEqual(self.bob_sprofile.nick, post_data['nick'])
        self.assertEqual(self.bob_sprofile.phrase, post_data['phrase'])

    def test_setup_profile_not_optin(self):
        """ test user consent to receive emails """
        client = Client()
        client.force_login(self.bob)
        url = reverse('social_layer:setup_profile')
        response = client.get(url)
        self.assertEqual(response.status_code, 200)
        post_data = {
            'nick': 'Bob Tester',
            'phrase': 'Testing this',
            'receive_email': '',
            }
        response = client.post(url, post_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.bob_sprofile.refresh_from_db()
        self.assertEqual(self.bob_sprofile.nick, post_data['nick'])
        self.assertEqual(self.bob_sprofile.phrase, post_data['phrase'])
        
    def test_social_login(self):
        """ test the login page defined in SOCIAL_VISITOR_LOGIN """
        client = Client()
        response = client.get(reverse('social_layer:social_login'), follow=True)
        self.assertEqual('/'+settings.SOCIAL_VISITOR_LOGIN, response.redirect_chain[0][0])
        
    def test_see_notifications(self):
        """ test the notifications page """
        self.test_write_comment()
        client = Client()
        client.force_login(self.alice)
        response = client.get(reverse('social_layer:view_notifications'))
        self.assertIn('Bob', str(response.content))
        self.assertEqual(response.status_code, 200)
        notif = Notification.objects.get(to=self.alice)
        self.assertTrue(notif.read)

    def test_like_comment(self):
        """ Hit the like button """
        self.test_write_comment()
        client = Client()
        client.force_login(self.alice)
        comment = Comment.objects.all()[0]
        response = client.get(reverse('social_layer:like_comment',
                                      kwargs={'pk': comment.pk,
                                              'didlike': 'like'}))
        self.assertEqual(response.status_code, 302)
        like = LikeComment.objects.get(user=self.alice)
        self.assertTrue(like.like)
        comment.refresh_from_db()
        self.assertEqual(comment.count_likes, 1)
        self.assertEqual(comment.count_dislikes, 0)
        

    def test_dislike_comment(self):
        """ Hit the dislike button """
        self.test_write_comment()
        client = Client()
        client.force_login(self.alice)
        comment = Comment.objects.all()[0]
        response = client.get(reverse('social_layer:like_comment',
                                      kwargs={'pk': comment.pk,
                                              'didlike': 'dislike'}))
        self.assertEqual(response.status_code, 302)
        like = LikeComment.objects.get(user=self.alice)
        self.assertFalse(like.like)
        comment.refresh_from_db()
        self.assertEqual(comment.count_likes, 0)
        self.assertEqual(comment.count_dislikes, 1)

    def test_set_profile_photo(self):
        """ test setting up a profile picture """
        client = Client()
        client.force_login(self.bob)
        post_data = {
            'nick': 'Bob Tester',
            'phrase': 'Testing this',
            'receive_email': 'on',
            'picture': SimpleUploadedFile("image.png", b64decode(small_image),
                                        content_type="image/png"),
            }
        url = reverse('social_layer:setup_profile')
        response = client.post(url, post_data, follow=True)
        self.assertEqual(response.status_code, 200)
        format_medias()
        self.bob_sprofile.refresh_from_db()
        self.assertEqual(self.bob_sprofile.nick, post_data['nick'])
        self.assertEqual(self.bob_sprofile.phrase, post_data['phrase'])
        self.assertIsNotNone(self.bob_sprofile.picture())
        self.assertTrue(os.path.isfile(self.bob_sprofile.picture().media_file.path))
        self.assertTrue(os.path.isfile(self.bob_sprofile.picture().media_thumbnail.path))
        self.assertEqual(self.bob_sprofile.picture().content_type, "image/png")

    def test_delete_profile_photo(self):
        self.test_set_profile_photo()
        client = Client()
        client.force_login(self.bob)
        self.bob_sprofile.refresh_from_db()
        pic = self.bob_sprofile.picture()
        url = reverse('social_layer:delete_profile_photo')
        response = client.get(url)
        self.bob_sprofile._cached_profoto = None # force refresh
        self.bob_sprofile.refresh_from_db()
        self.assertIsNone(self.bob_sprofile.picture())
        self.assertFalse(os.path.isfile(pic.media_file.path))
        self.assertFalse(os.path.isfile(pic.media_thumbnail.path))

    def test_task_photos(self):
        self.test_set_profile_photo()
        format_medias()

    def test_set_video_as_photo(self):
        """ test setting up a profile picture """
        client = Client()
        client.force_login(self.bob)
        post_data = {
            'nick': 'Bob Tester',
            'phrase': 'Testing this',
            'receive_email': 'on',
            'picture': SimpleUploadedFile("video.mp4", b64decode(small_video),
                                        content_type="video/mp4"),
            }
        url = reverse('social_layer:setup_profile')
        response = client.post(url, post_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.bob_sprofile.refresh_from_db()
        format_medias()
        self.assertEqual(self.bob_sprofile.nick, post_data['nick'])
        self.assertEqual(self.bob_sprofile.phrase, post_data['phrase'])
        self.assertIsNotNone(self.bob_sprofile.picture())
        self.assertTrue(os.path.isfile(self.bob_sprofile.picture().media_file.path))
        self.assertTrue(os.path.isfile(self.bob_sprofile.picture().media_thumbnail.path))
        self.assertEqual(self.bob_sprofile.picture().content_type, "video/mp4")

    # Posts
    def test_write_post(self):
        """ write a simple text post """
        client = Client()
        client.force_login(self.bob)
        post_data = {
            'text': 'gonna have a sandwich',
        }
        url = reverse('social_layer:new_post')
        response = client.post(url, post_data, follow=True)
        post = Post.objects.all().last()
        self.assertEqual(post.text, post_data['text'])
        self.assertIn(post_data['text'], str(response.content))
        response = client.get(post.get_url())
        self.assertEqual(response.status_code, 200)
        self.assertIn(post_data['text'], str(response.content))

    def test_delete_post(self):
        """ """
        self.test_write_post()
        client = Client()
        client.force_login(self.bob)
        post = Post.objects.all().last()
        client = Client()
        client.force_login(self.bob)
        url = reverse('social_layer:delete_post', kwargs={'pk': post.pk})
        response = client.get(url)
        self.assertIsNone(Post.objects.filter(pk=post.pk).last())

    @override_settings(SOCIAL_ALLOW_MEDIA_POSTS=True)
    def test_write_post_with_file(self):
        """ write a post with media file """
        from social_layer.posts.forms import PostForm
        PostForm.allow_media = True # monkey_patch
        client = Client()
        client.force_login(self.bob)
        post_data = {
            'text': 'this is my cat',
            'media': SimpleUploadedFile("cat.png", b64decode(small_image),
                                        content_type="image/png"),
        }
        url = reverse('social_layer:new_post')
        response = client.post(url, post_data, follow=True)
        post = Post.objects.all().last()
        self.assertEqual(post.text, post_data['text'])
        self.assertIn(post_data['text'], str(response.content))
        format_medias()
        self.assertIsNotNone(post.postmedia)
        post_media = post.postmedia
        self.assertIn(post.postmedia.media_thumbnail.url, str(response.content))
        self.assertIn('.webp', post.postmedia.media_thumbnail.url)
        self.assertTrue(os.path.isfile(post.postmedia.media_thumbnail.path))
        # delete_post
        url = reverse('social_layer:delete_post', kwargs={'pk': post.pk})
        response = client.get(url)
        self.assertIsNone(Post.objects.filter(pk=post.pk).last())
        self.assertIsNone(PostMedia.objects.filter(pk=post_media.pk).last())
        self.assertFalse(os.path.isfile(post.postmedia.media_file.path))
        self.assertFalse(os.path.isfile(post.postmedia.media_thumbnail.path))
    
    @override_settings(SOCIAL_ALLOW_MEDIA_POSTS=False)
    def test_write_post_with_file_denied(self):
        """ write a post with media file, but its not allowed :( """
        from social_layer.posts.forms import PostForm
        PostForm.allow_media = False # monkey_patch
        client = Client()
        client.force_login(self.bob)
        post_data = {
            'text': 'this is my cat',
            'media': SimpleUploadedFile("cat.png", b64decode(small_image),
                                        content_type="image/png"),
        }
        url = reverse('social_layer:new_post')
        self.assertIsNone(Post.objects.all().last())
        response = client.post(url, post_data, follow=True)
        post = Post.objects.all().last()
        self.assertIsNotNone(post)
        #self.assertIsNone(post.postmedia)

    def test_write_post_empty(self):
        """ write a empty post. not allowed :( """
        client = Client()
        client.force_login(self.bob)
        response = client.post(reverse('social_layer:new_post'),
                               {'text':''}, follow=True)
        post = Post.objects.all().last()

    def test_posts_feed_scroll(self):
        """ test the view more_posts, used by infscroll module """
        self.test_write_post()
        client = Client()
        post = Post.objects.all().last()
        url = reverse('social_layer:more_posts')
        response = client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertIn(post.text, str(response.content))




# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.utils import timezone

class LikableObject(models.Model):
    """ An object that people might hit a like button on
    Used by Comment and CommentSection
    """
    count_replies = models.PositiveIntegerField(default=0)
    count_likes = models.PositiveIntegerField(default=0)
    count_dislikes = models.PositiveIntegerField(default=0)

    class Meta():
        abstract = True


class Like(models.Model):
    """ an object that records when someone hits the Like button. It also
    represents the Dislike action.
    """
    user = models.ForeignKey('auth.User',
                             related_name='likecomment_user_%(class)ss',
                             on_delete=models.CASCADE)
    like = models.BooleanField(default=True)
    date_time = models.DateTimeField(default=timezone.localtime)

    class Meta():
        abstract = True



# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

from django.shortcuts import render, redirect
from django.contrib.auth.decorators import login_required
from django.contrib.auth import get_user_model
from django.contrib.admin.views.decorators import staff_member_required
from social_layer.notifications.models import Notification

@login_required
def view_notifications(request):
    """ List the past notifications """
    list_notif = Notification.objects.filter(to=request.user).order_by('-date_time')
    data = {
        'list_notif': list_notif,
        'has_notif': False,
        }
    response = render(request, 'social_layer/notifications/view_notifications.html', data)
    list_notif.filter(read=False).update(read=True)
    return response

def get_notifications(request):
    """ Returns a list of unread notifications of logged in user """
    if request.user.is_authenticated:
        notifs = Notification.objects.filter(to=request.user, read=False)
    else:
        notifs = Notification.objects.none()
    return notifs

@staff_member_required
def admin_send_notification(request, pk):
    user = get_user_model().objects.get(pk=pk)
    if request.method == "POST":
        notif = Notification.objects.create(to=user,
                                            text=request.POST.get('text'))
    list_notifs = Notification.objects.filter(to=user).order_by('-id')
    data = {'list_notif': list_notifs,}
    return render(request, 'social_layer/notifications/admin_send_notification.html', data)



from django.db import models
from django.utils import timezone
from django.urls import reverse
from social_layer.mediautils.models import Media
from social_layer.comments.models import CommentSection

class Post(models.Model):
    """ a Post is the class that defines a User Generated Content.
    Talking about social media websites, a post usually consist of:
        selfies, cat photos, gym session photos, photos os dishes, etc.
    """
    text = models.TextField(null=True, blank=True)
    date_time = models.DateTimeField(default=timezone.localtime)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
    
    owner = models.ForeignKey('social_layer.SocialProfile',
                              related_name='post_commentsection_owner',
                              on_delete=models.CASCADE)
    comments = models.ForeignKey('social_layer.CommentSection',
                                 related_name='post_commentsection',
                                 on_delete=models.CASCADE)
    # cached values
    _cached_get_url = None

    def save(self, *args, **kwargs):
        """ before saving: - set up a commentsection object. """
        if self.comments_id is None:
            self.comments = CommentSection.objects.create(owner=self.owner)
        super(Post, self).save(*args, **kwargs)
        if self.comments.get_url() != self.get_url():
            self.comments.url = self.get_url()
            self.comments.save()

    def get_url(self):
        """ get the url of this post """
        if self._cached_get_url is None:
            self._cached_get_url = reverse('social_layer:view_post',
                                           kwargs={'pk': self.pk})
        return self._cached_get_url

class PostMedia(Media):
    """ PostMedia holds the file associated with a POST.
    This is where cat pictures and gym photos will be stored.
    """
    post = models.OneToOneField('social_layer.Post', on_delete=models.CASCADE)




# Generated by Django 2.2.10 on 2020-12-02 19:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('social_layer', '0002_socialprofile_last_actv'),
    ]

    operations = [
        migrations.CreateModel(
            name='LikeComment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('like', models.BooleanField(default=True)),
                ('date_time', models.DateTimeField(default=django.utils.timezone.localtime)),
                ('comment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='like_comment', to='social_layer.Comment')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likecomment_user', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 2.2.10 on 2020-11-18 20:22

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import social_layer.mediautils.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CommentSection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.TextField(blank=True, null=True)),
                ('comments_enabled', models.BooleanField(default=True)),
                ('owner_can_delete', models.BooleanField(default=False)),
                ('anyone_can_reply', models.BooleanField(default=True)),
                ('is_flat', models.BooleanField(default=True)),
                ('count_replies', models.PositiveIntegerField(default=0)),
                ('count_likes', models.PositiveIntegerField(default=0)),
                ('count_dislikes', models.PositiveIntegerField(default=0)),
                ('cached_featured', models.TextField(blank=True, default='')),
            ],
        ),
        migrations.CreateModel(
            name='SocialProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nick', models.CharField(blank=True, max_length=64, null=True)),
                ('phrase', models.CharField(blank=True, max_length=256, null=True)),
                ('date_time', models.DateTimeField(default=django.utils.timezone.localtime)),
                ('ip', models.CharField(blank=True, max_length=46, null=True)),
                ('comment_section', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='sprofile_comment_section', to='social_layer.CommentSection')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='SocialProfilePhoto',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('media_file', models.FileField(max_length=10485760, upload_to=social_layer.mediautils.models.choose_upload_media_to)),
                ('media_thumbnail', models.FileField(blank=True, max_length=10485760, null=True, upload_to=social_layer.mediautils.models.choose_upload_media_thumb_to)),
                ('content_type', models.CharField(blank=True, max_length=127, null=True)),
                ('orientation', models.CharField(default='portrait', max_length=10)),
                ('format_tries', models.IntegerField(default=0)),
                ('formated', models.BooleanField(default=False)),
                ('md5_hash', models.CharField(blank=True, max_length=32, null=True)),
                ('profile', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='social_layer.SocialProfile')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('date_time', models.DateTimeField(default=django.utils.timezone.localtime)),
                ('read', models.BooleanField(default=False)),
                ('to', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notification_to', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='commentsection',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comment_section_owner', to='social_layer.SocialProfile'),
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('date_time', models.DateTimeField(default=django.utils.timezone.localtime)),
                ('count_replies', models.PositiveIntegerField(default=0)),
                ('count_likes', models.PositiveIntegerField(default=0)),
                ('count_dislikes', models.PositiveIntegerField(default=0)),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comment_author', to='social_layer.SocialProfile')),
                ('comment_section', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comment_section', to='social_layer.CommentSection')),
                ('reply_to', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='comment_reply_to', to='social_layer.Comment')),
            ],
            options={
                'ordering': ['-date_time'],
            },
        ),
    ]



XXXXXXX
XXXXXXXXXXX
XXXXXXXXXX
XXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


from django.db import models
from django.utils import timezone
# Create your models here.
class Notification(models.Model):
    """ Notification object, people get notified of social interactions """
    to = models.ForeignKey('auth.User',
                           related_name='notification_to',
                           on_delete=models.CASCADE)
    text = models.TextField()
    date_time = models.DateTimeField(default=timezone.localtime)
    read = models.BooleanField(default=False)
    
    def save(self, *args, **kwargs):
        other = Notification.objects.exclude(pk=self.pk
                                ).filter(to=self.to,
                                        text=self.text,
                                        date_time__date=self.date_time.date())
        if not other.exists():
            super(Notification, self).save(*args, **kwargs)




#import uuid
from django.db import models
from django.utils import timezone
from django.urls import reverse
from social_layer.mediautils.models import Media

# Create your models here.
class SocialProfile(models.Model):
    """ the Social media Profile object for the user to show online """
    user = models.OneToOneField('auth.User', on_delete=models.CASCADE)
    nick = models.CharField(max_length=64, null=True, blank=True)
    phrase = models.CharField(max_length=256, null=True, blank=True)

    date_time = models.DateTimeField(default=timezone.localtime)
    last_actv = models.DateTimeField(default=timezone.localtime)
    ip = models.CharField(max_length=46, null=True, blank=True)

    comment_section = models.OneToOneField('social_layer.CommentSection',
                                           related_name='sprofile_comment_section',
                                           on_delete=models.SET_NULL,
                                           null=True, blank=True)
    _cached_profoto = None

    def save(self, *args, **kwargs):
        """ the save takes care of setting up a nickname and fix string sizes
        """
        if not self.nick:
            if '@' in self.user.username:
                self.nick = self.user.username.split('@')[0]
            else:
                self.nick = self.user.username
        if self.nick:
            self.nick = self.nick[0:64]
        if self.phrase:
            self.phrase = self.phrase[0:256]
        super(SocialProfile, self).save(*args, **kwargs)

    def picture(self):
        """ return the profile thumbnail """
        if self._cached_profoto is None:
            self._cached_profoto = SocialProfilePhoto.objects.filter(
                                                    profile=self).last()
        return self._cached_profoto

    def get_url(self):
        """ get the url to the profilepage """
        return reverse('social_layer:view_profile', kwargs={'pk':self.pk})
    
class SocialProfilePhoto(Media):
    """ picture used by the SocialProfile.picture
    """
    profile = models.ForeignKey(SocialProfile, on_delete=models.CASCADE)




# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

from importlib import import_module
from social_layer.profiles.models import SocialProfile, SocialProfilePhoto

def get_social_profile_byuser(user):
    """ returns a SocialProfile object given an user """
    sprofile = SocialProfile.objects.filter(user=user).first()
    if not sprofile:
        sprofile = SocialProfile.objects.create(user=user)
    return sprofile

def get_social_profile(request):
    """ Returns the social profile of the user """
    if request.user.is_authenticated:
        sprofile = get_social_profile_byuser(request.user) 
        if not sprofile.ip:
            sprofile.ip = (request.META.get('HTTP_X_FORWARDED_FOR', None)
                           or request.META.get('REMOTE_ADDR', None))
            sprofile.save()
        return sprofile
    return None

def execute_string(function_string, *args, **kwargs):
    """ executes a function given it name as a string """
    mod_name, func_name = function_string.rsplit('.',1)
    mod = import_module(mod_name)
    func = getattr(mod, func_name)
    result = func(*args, **kwargs)
    return result

    




# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

from django.shortcuts import render, redirect

# Create your views here.
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import Http404
from django.urls import reverse
from social_layer.mediautils.utils import handle_upload_file

from social_layer.utils import get_social_profile
from social_layer.comments.models import Comment
from social_layer.profiles.models import SocialProfile, SocialProfilePhoto
from social_layer.utils import execute_string

@login_required
def setup_profile(request):
    """ Setup the social profile settings """
    did_exist = SocialProfile.objects.filter(user=request.user).exists()
    sprofile = get_social_profile(request)
    alt_setup = getattr(settings, 'SOCIAL_ALT_SETUP_PROFILE', None)
    if alt_setup:
        # pass an alternative function if your app needs
        # this should should None if expects the normal behavior
        response = execute_string(alt_setup, request)
        if response:
            return response
    if request.method == "POST":
        sprofile.nick = request.POST.get('nick', '')
        sprofile.phrase = request.POST.get('phrase', '')
        for upload_file in request.FILES:
            foto = handle_upload_file(file_post=request.FILES[upload_file],
                                        quality=2,
                                        Model=SocialProfilePhoto,
                                        extra_args={'profile': sprofile})
            if foto:
                oldies = SocialProfilePhoto.objects.exclude(pk=foto.pk).filter(profile=sprofile)
                oldies.delete()
        sprofile.save()
        # TODO handle spam optin
        receive_email = bool(request.POST.get('receive_email', ''))
        redir_after = request.COOKIES.get('slogin_next', None)
        if redir_after:
            resp = redirect(redir_after)
            resp.delete_cookie('slogin_next')
            return resp
        else:
            return redirect(sprofile.get_url())
    data = {
        'sprofile': sprofile,
        'did_exist': bool(did_exist),
        }
    return render(request, 'social_layer/profiles/setup_profile.html', data)
    
def view_profile(request, pk):
    """ View the User's profile page """
    sprofile = SocialProfile.objects.filter(pk=pk).first()
    if not sprofile:
        raise Http404()
    alt_view = getattr(settings, 'SOCIAL_ALT_VIEW_PROFILE', None)
    if alt_view:
        # pass an alternative function if your app needs.
        # this should return None if excpets the normal behavior
        response = execute_string(alt_view, request, sprofile)
        if response:
            return response
    data = {
        'sprofile': sprofile,
        'comments': Comment.objects.filter(author=sprofile)
        }
    # a callable that returns a dict for the social profile view
    if getattr(settings, 'SOCIAL_PROFILE_CONTEXT', None):
        extra = execute_string(settings.SOCIAL_PROFILE_CONTEXT, sprofile)
        data.update(extra)
    return render(request, 'social_layer/profiles/profile.html', data)


def social_login(request):
    """ 
    The Implementing application MUST take care of authentication.
    Any action that requires a social login must be redirected here. 
    It will redirect the user to the 'social' login page.
    This url is defined by: settings.SOCIAL_VISITOR_LOGIN
    """
    next_url = request.GET.get('next', '/')
    resp = redirect('/'+settings.SOCIAL_VISITOR_LOGIN)
    resp.set_cookie('slogin_next', next_url, expires=360) 
    return resp


def list_profiles(request):
    """ List other people's profiles """
    list_profiles = SocialProfile.objects.all().order_by('?')
    data = {'list_profiles': list_profiles[0:100],}
    return render(request, 'social_layer/profiles/list_profiles.html', data)


def delete_profile_photo(request):
    """ The delete the user's photo """
    profile = get_social_profile(request)
    pic = profile.picture()
    if pic is not None:
        pic.delete()
    #post = get_object_or_404(SocialProfilePhoto,
                             #pk=pk, profile__user=request.user)
    #post.delete()
    return redirect(reverse('social_layer:setup_profile'))
 



import os

import logging
from django.conf import settings
from social_layer.mediautils.utils import (cropa_imagem, cropa_video,
                                           convert_towebp,
                                           get_thumb_from_video)

logger = logging.getLogger(__name__)

def formata_all_media(Model=None):
    """ Manipulate media files as needed
    """
    wsize = 480
    hsize = 360
    list_media = Model.objects.filter(formated=False)
    for post in list_media:
        logger.info("processing {} {}...".format(str(Model), str(post.pk)))
        if 'video/' in post.content_type:
            post.formated = cropa_video(post.media_file.path,
                                        larger=wsize, smaller=hsize)
            get_thumb_from_video(post)
            if post.media_thumbnail:
                convert_towebp(post.media_thumbnail.path)
            post.save()
        elif 'image/gif' in post.content_type:
            cropa_imagem(post.media_thumbnail.path, quality=1)
            convert_towebp(post.media_thumbnail.path)
            post.formated = True
            post.save()
        elif 'image/' in post.content_type:
            if post.media_file:
                cropa_imagem(post.media_file.path, quality=2)
            if post.media_thumbnail:
                cropa_imagem(post.media_thumbnail.path, quality=1)
                convert_towebp(post.media_thumbnail.path)
            post.formated = True
            post.save()
        else: # we dont process other kind of files
            post.formated = True
            post.save()



# This file is part of django-social-layer
#
#    django-social-layer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    django-social-layer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with django-social-layer. If not, see <http://www.gnu.org/licenses/>.

import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-social-layer',
    version='0.2.4',
    packages=find_packages(),
    include_package_data=True,
    license='GNU License',  # example license
    description='Adds social media features to any website',
    long_description=README,
    long_description_content_type='text/x-rst',
    url='https://github.com/gsteixeira',
    author='Gustavo Selbach Teixeira',
    author_email='gsteixei@gmail.com',
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        ],
    install_requires=[
        'Django>=2.2.0',
        'Pillow<=8.4.0',
        'opencv-python<=4.5.4.60',
        'django-infinite-scroll>=0.1.7',
        ],
    package_data={
        'social_layer': ['social_layer/migrations/*',]
        },
)




import os
from django.dispatch import receiver
from django.db.models.signals import post_save, post_delete

from social_layer.posts.models import PostMedia
from social_layer.profiles.models import SocialProfilePhoto

@receiver(post_delete, sender=PostMedia)
@receiver(post_delete, sender=SocialProfilePhoto)
def post_media_post_delete(**kwargs):
    instance = kwargs.get('instance')
    try:
        os.remove(instance.media_file.path)
    except FileNotFoundError:
        pass
    try:
        os.remove(instance.media_thumbnail.path)
    except FileNotFoundError:
        pass

    



# Generated by Django 3.0.6 on 2020-11-21 15:08

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('social_layer', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='socialprofile',
            name='last_actv',
            field=models.DateTimeField(default=django.utils.timezone.localtime),
        ),
    ]



#from django.db import models
import uuid
from django.db import models
from django.conf import settings
from django.utils import timezone
from django.urls import reverse
from social_layer.models import Like, LikableObject

### Comments
class CommentSection(LikableObject):
    """ A comment section can be added to any page.
    It optionally is tied to a url. But also can be refered by its id.
    the owner is also optional. If is not None, the owner will get a
    notification when someone makes a new comment.
    """
    url = models.TextField(null=True, blank=True)
    owner = models.ForeignKey('social_layer.SocialProfile',
                              related_name='comment_section_owner',
                              null=True, blank=True,
                              on_delete=models.CASCADE)
   
    comments_enabled = models.BooleanField(default=True)
    owner_can_delete = models.BooleanField(default=False)
    anyone_can_reply = models.BooleanField(default=True)
    is_flat = models.BooleanField(default=True)

    cached_featured = models.TextField(blank=True, default="")
    # in case of featured = True, it will show limited comments
    featured = False
    _cached_comment_list = None
    max_comments = getattr(settings, 'SOCIAL_MAX_FEATURED_COMMENTS', 3)
    
    def get_url(self):
        """ return the parent url of the comment or the default view """
        if self.url:
            return self.url
        else:
            return reverse('social_layer:comment_section',
                           kwargs={'pk':self.pk})

    def get_comments(self):
        """ return comments from this comment section
        You can limit the amount of comments with the 'featured' flag.
        If True, only a number of MAX_FEATURED_COMMENTS (defaults to 3) will be
        rendered on screen.
        """
        if self._cached_comment_list is None:
            self._cached_comment_list = Comment.objects.filter(
                                                comment_section=self,
                                                reply_to=None)
        if self.featured:
            return self._cached_comment_list[0:self.max_comments]
        else:
            return self._cached_comment_list

    def get_featured_comments(self):
        """ get a limited number of comments, defined by MAX_FEATURED_COMMENTS
        """
        max_comm = getattr(settings, 'SOCIAL_MAX_FEATURED_COMMENTS', 3)
        return self.get_comments()[0:max_comm]

    def updt_counters(self):
        """ Updates the counting """
        self.count_likes = LikePost.objects.filter(comment_section=self,
                                                   like=True).count()
        self.count_dislikes = LikePost.objects.filter(comment_section=self,
                                                      like=False).count()
        self.count_replies = Comment.objects.filter(comment_section=self).count()
        self.save()


class Comment(LikableObject):
    """ the Comment object """
    comment_section = models.ForeignKey('social_layer.CommentSection',
                                        related_name='comment_section',
                                        on_delete=models.CASCADE)
    author = models.ForeignKey('social_layer.SocialProfile',
                               related_name='comment_author',
                               on_delete=models.CASCADE)
    text = models.TextField()
    date_time = models.DateTimeField(default=timezone.localtime)
    reply_to = models.ForeignKey('social_layer.Comment',
                                related_name='comment_reply_to',
                                on_delete=models.CASCADE,
                                null=True, blank=True)

    class Meta():    
        ordering = ['-date_time']

    def save(self, *args, **kwargs):
        """ the save method prevents repeated comments and crops the text.
        """
        if self.text:
            self.text = self.text.replace('\n', ' ')[0:512]
            other = Comment.objects.exclude(pk=self.pk
                                  ).filter(author=self.author,
                                           text=self.text,
                                           comment_section=self.comment_section,
                                           reply_to=self.reply_to)
            if not other.exists():
                super(Comment, self).save(*args, **kwargs)
                self.comment_section.updt_counters()

    def get_replies(self):
        """ Return all the replies to this comment """
        if not hasattr(self, 'cached_get_replies'):
            replies = Comment.objects.filter(comment_section=self.comment_section,
                                             reply_to=self)
            self.cached_get_replies = replies
        return self.cached_get_replies

    def updt_counters(self):
        """ Updates the counting """
        self.count_likes = LikeComment.objects.filter(comment=self,
                                                      like=True).count()
        self.count_dislikes = LikeComment.objects.filter(comment=self,
                                                         like=False).count()
        self.count_replies = Comment.objects.filter(reply_to=self).count()
        self.save()


class LikeComment(Like):
    """ a Like for a Comment """
    comment = models.ForeignKey('social_layer.Comment', 
                                related_name='like_comment',
                                on_delete=models.CASCADE)

class LikePost(Like):
    """ a Like for a CommentSection """
    comment_section = models.ForeignKey('social_layer.CommentSection', 
                                        related_name='like_commentsection',
                                        on_delete=models.CASCADE)





from social_layer.mediautils.tasks import formata_all_media
from social_layer.profiles.models import SocialProfilePhoto
from social_layer.posts.models import PostMedia

def format_medias():
    formata_all_media(SocialProfilePhoto)
    formata_all_media(PostMedia)
    
